@@ -21,7 +21,7 @@ $evh->use_socket($socket);
 $evh->request_handler(sub {
     my $r = shift;
     my $n = "only";
-    my $w = $r->start_streaming(200, [
+    my $w = $r->start_streaming("200 OK", [
         'Content-Type' => 'text/plain',
         'Connection' => 'close',
     ]);
